/*
 * $Header: /home/cvspublic/jakarta-slide/src/share/org/apache/slide/macro/DeleteListener.java,v 1.4.2.1 2004/02/05 16:05:08 mholz Exp $
 * $Revision: 1.4.2.1 $
 * $Date: 2004/02/05 16:05:08 $
 *
 * ====================================================================
 *
 * Copyright 1999-2002 The Apache Software Foundation 
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 */
package org.apache.slide.macro;

import org.apache.slide.common.SlideException;

/**
 * A DeleteListener may be handed to the Macro helper in order to have more
 * control on the delete operation. The DeleteListener will be notified
 * before and after deleting any single resource that is target of the
 * recursive delete operation.
 *
 * @version $Revision: 1.4.2.1 $
 *
 * @author <a href="mailto:ralf.stuckert@softwareag.com">Ralf Stuckert</a>
 **/
public interface DeleteListener {
    
    /**
     * This method is called prior to deleting the resource associated by
     * the given <code>targetUri</code>. The deletion can be prohibited by
     * throwing a SlideException.
     *
     * @param      targetUri       the Uri of the resource that will be deleted.
     *
     * @throws     SlideException  this Exception will be passed to the caller
     *                             of the Macro helper (contained in the
     *                             MacroDeleteException.
     */
    public void beforeDelete(String targetUri) throws SlideException;
    
    /**
     * This method is called after deleting the resource associated by
     * the given <code>targetUri</code>.
     *
     * @param      targetUri       the Uri of the resource that has been deleted.
     *
     * @throws     SlideException  this Exception will be passed to the caller
     *                             of the Macro helper (contained in the
     *                             MacroDeleteException.
     */
    public void afterDelete(String targetUri) throws SlideException;
    
}

